# import stuff 
from Products.ZSyncer import ZSyncer

# import stuff
from Globals import DTMLFile, MessageDialog, Persistent, data_dir
from AccessControl import getSecurityManager
from StringIO import StringIO

# some other stuff
import OFS.SimpleItem
import Acquisition
import AccessControl.Role
import string, base64, sys, os, time

# do xml via basic auth, yeah!
import xmlrpclibBasicAuth

# needed for diff
from OFS.History import html_diff, replace
from string import join, split, atoi, strip
from DocumentTemplate.DT_Util import html_quote
from DocumentTemplate.DT_Var import url_quote_plus
from types import StringType

#from AccessControl import ClassSecurityInfo



manage_addServerForm = DTMLFile('dtml/Add', globals()) 

def manage_addServer(self, id='', title='', REQUEST=None): 
    """ adds a server """ 
    self._setObject(id, ZSyncerMute(id, title)) 
    if REQUEST is not None: 
        return MessageDialog(title = 'Added', message = "ZSyncerMute sucessfully added.", action = 'manage_main', ) 


class ZSyncerMute(ZSyncer.ZSyncer): 
    meta_type = 'ZSyncerMute'

    __ac_permissions__= (
        ('View management screens', (
        'cp', 'rcp',
        'manage_tabs','manage_main','is_diffable', 'manage_approvedAction', 'manage_addXMLRPC',
        'manage_listObjects', 'manage_exportXMLRPC', 'manage_deleteXMLRPC', 'manage_diffObject',
        'manage_compare', 'manage_srcXMLRPC', 'status_colour',) ),
        ('Change permissions', ('manage_access',) ),
    )



    """ Opens a conexion and executes cp on the remote server """
    """ Return 500 if fails """
    """ Return 200 if succeed """
    def rcp(self, remote_path, local_path,type=None):
          msgs = [] 
          for srv in self.dest_server:
             serverconn = self._serverConn(self._check_http(srv)) 
             try:
                 result = serverconn.cp(remote_path,local_path,type) 
             except: 
                 msgs.append('Remote connexion to server %s failed' % srv)
                 self._msg(msgs)
                 return 500
             msgs.append('Remote connexion to server %s successfull' % srv)
             self._msg(msgs)
          return(result)



    """ Copy all the objects with meta-type='type' in folder 'local_path' to the remote folder 'remote_path' """
    """ 'type'==None -> All the objects """ 
    """ Return 500 if fails """
    """ Return 200 if succeed """
    def cp(self,local_path,remote_path,type=None):
        lista=[]
        msgs = [] 
        local_folder= self.unrestrictedTraverse(local_path)
        # The user must must have the permission
        # ""
        c = getSecurityManager().checkPermission
        allowed = 1
        for perm in ['Delete objects', 'Import/Export objects']:
            if not c(perm, local_folder): allowed = 0
        if not allowed: return 403

        for object in local_folder.objectValues(type):
               path_object=string.join(object.getPhysicalPath(),'/')
               msgs.append('cp: found object %s with type %s' % (path_object, type)) 
               lista.append(path_object)
        return(self.manage_exportXMLRPC(lista,msgs,remote_path))


    """ remote_path = [ '', 'el', 'path', 'completo'] """
    def manage_exportXMLRPC(self, object, msgs=None, remote_path=None): 
        """ Interface to calling export on each object to destination server via XMLRPC """ 
        result=200
        lastbadresult=200
        if msgs is None: msgs = [] 
        if type(object) == StringType: object = [object,]  
        for ob in object: 
            for srv in self.dest_server: 
                # more logging 
                msgs.append('Local object %s ' % ob) 
                msgs.append('Syncing to remote server %s ' % srv)
                result=self._exportXMLRPC(ob, srv,1,remote_path)
                if result==500:
                   lastbadresult=result
                   msgs.append('Error copying object %s' % ob)
                   msgs.append('to server %s ' % srv)
                self._msg(msgs)
        return lastbadresult




    def _exportXMLRPC(self, object=None, dest_url=None, add_in=1, remote_path=None): 
        """exports an object""" 
        # go get the object
        obj = self.unrestrictedTraverse(object)
        data = StringIO()
        obj._p_jar.exportFile(obj._p_oid, data)
        # do xml-rpc
        if dest_url[:8] == 'https://':
           serverconn = self._serverSecureConn(self._check_http(dest_url))
        else:
           serverconn = self._serverConn(self._check_http(dest_url))

        if remote_path is None:
              path=obj.getPhysicalPath()
        else:
               """ Substitute the source path with remote_path """
               path=string.join(remote_path,'/')+'/'+(string.split(object, '/'))[-1]
             
#        try:
        result = serverconn.manage_addXMLRPC(self._encode(data.getvalue()), path, add_in)
#        except: return 500 
 
        # return result 
        return result

    def _serverSecureConn(self, server):
        import M2Crypto.m2xmlrpclib
        # connect using m2xmlrpc lib
        t = M2Crypto.m2xmlrpclib.SSL_Transport()
        return M2Crypto.m2xmlrpclib.Server(server, t)

    def _check_http(self, v):
        # so people can get away with leaving http:// off :)
        if v[:7] != 'http://' and v[:8]!='https://' : v = 'http://%s' % v
        return v
    

    def _msg(self, msgs):
        # what to do with these messages
        ms = []
        for m in msgs: 
            # hack-o-rama, if i have a number, its a status code, so lets look it up and get a nicer message
            if type(m) == type(1): ms.append('%s: %s' % (m, self._error_messages.get(str(m), 'Unknown error occured')))
            else: ms.append(m)

        if self.log or self.approval: self._log(ms)



    def manage_addXMLRPC(self, data, obj_path, add_in=1):
        """ Adds an object into the server """
        # make sure we always get a string or a list
        if type(obj_path) == type('string'): obj_path = string.split(obj_path, '/')

        # object lets try finding the parent
        try: parent = self.unrestrictedTraverse(obj_path[:-1])
        except KeyError: return 404

        # lets check they are allowed to do this
        c = getSecurityManager().checkPermission
        allowed = 1
        for perm in ['Delete objects', 'Import/Export objects']:
            if not c(perm, parent): allowed = 0
        if not allowed: return 403

        # if there is one there already, delete it
        if obj_path[-1] in parent.objectIds(): 
            parent.manage_delObjects([obj_path[-1],])
        else:
            if not add_in: # ok so if we are deleting
                return 404

        # lets do it
        if add_in:
            # fake a file using StringIO
            file = StringIO()
            file.write(self._decode(data))
            file.seek(0)

            # now import
            new = parent._p_jar.importFile(file)
            parent._setObject(new.getId(), new)

        # yay!
        return 200
